RESOLVE_PREFIX = 'resolve_'
POST_PREFIX = 'post_'
PYDANTIC_FORWARD_REF_UPDATED = '__pydantic_resolve_forward_refs_updated__'
HAS_MAPPER_FUNCTION = '__pydantic_resolve_mapper_provided__'
POST_DEFAULT_HANDLER = 'post_default_handler'

EXPOSE_TO_DESCENDANT = '__pydantic_resolve_expose__'
COLLECTOR_CONFIGURATION = '__pydantic_resolve_collect__'

ENSURE_SUBSET_REFERENCE = '__pydantic_resolve_ensure_subset_reference__'
ENSURE_SUBSET_DEFINITION = '__pydantic_resolve_subset__'
ENSURE_SUBSET_DEFINITION_SHORT = '__subset__'

ER_DIAGRAM = '__pydantic_resolve_er_diagram__'
ER_DIAGRAM_PRE_GENERATOR = '__pydantic_resolve_er_diagram_pre_gen__'
ER_DIAGRAM_INLINE_RELATIONSHIPS = '__pydantic_resolve_relationships__'
ER_DIAGRAM_INLINE_RELATIONSHIPS_SHORT = '__relationships__'
